@@ -14,7 +14,7 @@
 #define SETUP_CONFIG                                                \
 	{                                                           \
 		int rc;                                             \
-		rc = config__init(&f->config, 1, "1", NULL, "dir"); \
+		rc = config__init(&f->config, 1, "1", NULL); \
 		munit_assert_int(rc, ==, 0);                        \
 		test_logger_setup(params, &f->config.logger);       \
 	}
