@@ -1,5 +1,5 @@
+#include "../../lib/runner.h"
 #include "../lib/cluster.h"
-#include "../lib/runner.h"
 
 /******************************************************************************
  *
