@@ -1,6 +1,6 @@
 #include "../../../src/raft.h"
 
-#include "../lib/runner.h"
+#include "../../lib/runner.h"
 
 /******************************************************************************
  *
